import uuid

from datetime import date, datetime
from decimal import Decimal
from typing import List, Union

from pydantic import validator
from pydantic.main import BaseModel

from . import to_camel


class BatchIntegrityItem(BaseModel):
    date_: date
    type_: str
    url: List[str]
    quantity: Decimal
    price: Decimal

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel
        json_encoders = {date: lambda v: v.strftime("%d-%b-%Y")}

    @validator("date_", pre=True)
    def parse_start_date(cls, value: Union[date, str]) -> date:
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()


class BatchIntegrity(BaseModel):
    id_: uuid.UUID
    product: str
    date_: date
    price: Decimal
    showing: Decimal
    actual: Decimal
    details: List[BatchIntegrityItem]

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel
        json_encoders = {date: lambda v: v.strftime("%d-%b-%Y")}

    @validator("date_", pre=True)
    def parse_start_date(cls, value: Union[date, str]) -> date:
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()
