from datetime import date, datetime
from typing import Optional, Union

from pydantic import BaseModel, validator

from ..models.voucher_type import VoucherType
from . import to_camel
from .account import AccountLink
from .cost_centre import CostCentreLink


class BlankVoucherInfo(BaseModel):
    type_: VoucherType
    date_: date
    account: Optional[AccountLink]
    source: Optional[CostCentreLink]
    destination: Optional[CostCentreLink]

    class Config:
        alias_generator = to_camel
        json_encoders = {date: lambda v: v.strftime("%d-%b-%Y")}

    @validator("date_", pre=True)
    def parse_date(cls, value: Union[date, str]) -> date:
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()
