from datetime import date, datetime
from decimal import Decimal
from typing import List, Union

from pydantic import validator
from pydantic.main import BaseModel

from . import to_camel


class PurchaseEntriesItem(BaseModel):
    date_: date
    url: List[str]
    supplier: str
    product: str
    quantity: Decimal
    rate: Decimal
    tax: Decimal
    discount: Decimal
    amount: Decimal

    @validator("date_", pre=True)
    def parse_date(cls, value: Union[date, str]) -> date:
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel


class PurchaseEntries(BaseModel):
    start_date: date
    finish_date: date
    body: List[PurchaseEntriesItem]

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel
        json_encoders = {date: lambda v: v.strftime("%d-%b-%Y")}

    @validator("start_date", pre=True)
    def parse_start_date(cls, value: Union[date, str]) -> date:
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @validator("finish_date", pre=True)
    def parse_finish_date(cls, value: Union[date, str]) -> date:
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()
