import uuid

from datetime import date, datetime
from decimal import Decimal
from typing import List, Optional, Union

from pydantic import validator
from pydantic.main import BaseModel

from . import to_camel


class RawMaterialCostItem(BaseModel):
    name: Optional[str]
    issue: Optional[Decimal]
    sale: Optional[Decimal]
    rmc: Optional[Decimal]
    url: Optional[List[str]]

    group: Optional[str]
    quantity: Optional[Decimal]
    net: Optional[Decimal]
    gross: Optional[Decimal]
    order: int

    heading: Optional[bool]

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel


class RawMaterialCost(BaseModel):
    id_: Optional[uuid.UUID]
    start_date: date
    finish_date: date
    body: List[RawMaterialCostItem]
    footer: Optional[RawMaterialCostItem]

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel
        json_encoders = {date: lambda v: v.strftime("%d-%b-%Y")}

    @validator("start_date", pre=True)
    def parse_start_date(cls, value: Union[date, str]) -> date:
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @validator("finish_date", pre=True)
    def parse_finish_date(cls, value: Union[date, str]) -> date:
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()
