import { HttpClient, HttpParams } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/internal/Observable';
import { catchError } from 'rxjs/operators';

import { ErrorLoggerService } from '../core/error-logger.service';

import { ClosingStock } from './closing-stock';

const url = '/api/closing-stock';
const serviceName = 'ClosingStockService';

@Injectable({
  providedIn: 'root',
})
export class ClosingStockService {
  constructor(private http: HttpClient, private log: ErrorLoggerService) {}

  list(date: string | null, costCentre: string | null): Observable<ClosingStock> {
    const listUrl = date === null ? url : `${url}/${date}`;
    const options = { params: new HttpParams() };
    if (costCentre !== null) {
      options.params = options.params.set('d', costCentre);
    }
    return this.http
      .get<ClosingStock>(listUrl, options)
      .pipe(catchError(this.log.handleError(serviceName, 'list'))) as Observable<ClosingStock>;
  }

  save(closingStock: ClosingStock): Observable<ClosingStock> {
    return this.http
      .post<ClosingStock>(url, closingStock)
      .pipe(catchError(this.log.handleError(serviceName, 'save'))) as Observable<ClosingStock>;
  }

  post(date: string, costCentre: string): Observable<ClosingStock> {
    const options = { params: new HttpParams().set('d', costCentre) };
    return this.http
      .post<ClosingStock>(`${url}/${date}`, {}, options)
      .pipe(
        catchError(this.log.handleError(serviceName, 'Post Voucher')),
      ) as Observable<ClosingStock>;
  }

  delete(date: string, costCentre: string): Observable<ClosingStock> {
    const options = { params: new HttpParams().set('d', costCentre) };
    return this.http
      .delete<ClosingStock>(`${url}/${date}`, options)
      .pipe(
        catchError(this.log.handleError(serviceName, 'Delete Voucher')),
      ) as Observable<ClosingStock>;
  }
}
